from datetime import datetime

import nonebot
from nonebot import on_keyword
from nonebot import require
from nonebot.adapters.onebot.v11 import MessageSegment, PrivateMessageEvent
from nonebot.adapters.onebot.v11 import GROUP_ADMIN, GROUP_OWNER
from nonebot.permission import SUPERUSER

from .nlp_util import get_time_from_text, get_name_from_text, get_priority_from_text
from .todo import TodoToken, TodoUtil
from .config import Config

global_config = nonebot.get_driver().config
plugin_config = Config(**global_config.dict())

add_todo = on_keyword({'提醒', 'nonebot_todo'}, permission=SUPERUSER)
remove_todo = on_keyword({'删除', '去掉'}, permission=SUPERUSER)
finish_todo = on_keyword({'完成'}, permission=SUPERUSER)
change_todo_time = on_keyword({'更正', '改'}, permission=SUPERUSER)
get_todo_pic = on_keyword({'获取todo'}, permission=SUPERUSER)
scheduler = require("nonebot_plugin_apscheduler").scheduler


@add_todo.handle()
async def _handle(event: PrivateMessageEvent):
    text: str = event.raw_message
    exceed_time, time_valid, time_error_doc = get_time_from_text(text)
    name, name_valid, name_error_doc = get_name_from_text(text)
    priority, priority_valid, priority_error_doc = get_priority_from_text(text)
    if not time_valid:
        await add_todo.finish(time_error_doc)
    elif not name_valid:
        await add_todo.finish(name_error_doc)
    elif not priority_valid:
        await add_todo.finish(priority_error_doc)

    todo_util = TodoUtil(str(event.user_id))
    todo_token = TodoToken(name, datetime.today(), exceed_time, priority)
    if todo_util.add_data_to_list(todo_token):
        await add_todo.send(f"已将{name}加入清单，ddl为{exceed_time}，优先级为{priority}。\n当前共{todo_util.list_size}项待办。")
        img = await todo_util.get_list_img()
        await remove_todo.finish(MessageSegment.image(img))
    else:
        await add_todo.finish(f"{name}已在清单中。")


@remove_todo.handle()
async def _handle(event: PrivateMessageEvent):
    text: str = event.raw_message
    name, name_valid, name_error_doc = get_name_from_text(text)
    todo_util = TodoUtil(str(event.user_id))
    if not name_valid:
        await remove_todo.finish(name_error_doc)
    elif todo_util.remove_data(name) == 0:
        await remove_todo.finish(f"不存在名为{name}的表项。")
    else:
        img = await todo_util.get_list_img()
        await remove_todo.send(f"已删除{name}。")
        await remove_todo.finish(MessageSegment.image(img))


@finish_todo.handle()
async def _handle(event: PrivateMessageEvent):
    text: str = event.raw_message
    name, name_valid, name_error_doc = get_name_from_text(text)
    todo_util = TodoUtil(str(event.user_id))
    if not name_valid:
        await remove_todo.finish(name_error_doc)
    elif todo_util.finish_job(name) == 0:
        await remove_todo.finish(f"不存在名为{name}的表项。")
    else:
        await remove_todo.send(f"已完成{name}。加油！还剩{todo_util.list_size}项。")
        img = await todo_util.get_list_img()
        await remove_todo.finish(MessageSegment.image(img))


@change_todo_time.handle()
async def _handle(event: PrivateMessageEvent):
    text: str = event.raw_message
    name, name_valid, name_error_doc = get_name_from_text(text)
    expire_time, time_valid, time_error_doc = get_time_from_text(text)
    todo_util = TodoUtil(str(event.user_id))
    if not time_valid:
        await add_todo.finish(time_error_doc)
    elif not name_valid:
        await add_todo.finish(name_error_doc)
    else:
        todo_util.change_time(name, expire_time)
        await change_todo_time.send(f"将{name}改至{expire_time}")
        img = await todo_util.get_list_img()
        await change_todo_time.finish(MessageSegment.image(img))


@get_todo_pic.handle()
async def _handle(event: PrivateMessageEvent):
    todo_util = TodoUtil(str(event.user_id))
    img = await todo_util.get_list_img()
    await get_todo_pic.finish(MessageSegment.image(img))


async def send_todo():
    for qq in plugin_config.todo_qq_friends:
        todo_util = TodoUtil(str(qq))
        img = await todo_util.get_list_img()
        await nonebot.get_bot().send_private_msg(user_id=qq, message=MessageSegment.image(img))


for index, time in enumerate(plugin_config.todo_send_time):
    scheduler.add_job(send_todo, "cron", hour=time.hour, minute=time.minute, id=str(f"send_todo_for_{index}"))


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_todo_nlp']

package_data = \
{'': ['*'], 'nonebot_plugin_todo_nlp': ['templates/*', 'templates/css/*']}

install_requires = \
['jionlp>=1.4.17,<2.0.0',
 'nonebot-plugin-apscheduler>=0.1.4,<0.2.0',
 'nonebot-plugin-htmlrender>=0.1.1,<0.2.0',
 'nonebot2>=2.0.0-beta.5,<3.0.0',
 'pandas>=1.4.3,<2.0.0',
 'pathlib>=1.0.1,<2.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-todo-nlp',
    'version': '0.1.0',
    'description': '一款自动识别提醒内容，可生成todo图片并定时推送的nonebot插件',
    'long_description': '# nonebot_plugin_todo_nlp\n\n一款自动识别提醒内容，可生成todo图片并定时推送的nonebot插件\n\n消息响应器集合：add_todo, finish_todo,remove_todo,change_todo, get_todo_pic\n\n### 插件特点：\n\n* 允许多样化的日期描述方法，可以在语句中包含“明天”、“9月1日”等日期提示\n* 自动识别语句中的事项\n\n### 触发关键词：\n\n* 增加事项: \'提醒\', \'nonebot_todo\'\n* 完成事项: \'完成\'\n* 删除事项（支持正则）: \'删除\', \'去掉\'\n* 修改事项时间: \'更正\', \'改\'\n* 获取图片: \'获取todo\'\n\n> 若要强调事件名称（nlp有的时候会犯蠢，比如在示例中机器人没能识别出“中秋假期”这一关键词）：使用英文双引号括上事项名称\n\n### 推送配置方法：\n\n```\nTODO_QQ_FRIENDS=[864341840]\nTODO_SEND_TIME=[{"HOUR":08,"MINUTE":00}]\n```\n\n### TODO：\n- [ ] 迁移到postgre数据库\n- [ ] 增加优先级相关功能\n- [ ] 增加完成todo统计，对完成状况进行跟踪\n- [ ] 增加todo项目复用功能（比如每日/每周某天的提醒可以复用）\n\n### 示例：\n\n![1.png](https://github.com/CofinCup/nonebot_plugin_todo_nlp/blob/master/readme_resource/1.png)\n\n![2.jpg](https://github.com/CofinCup/nonebot_plugin_todo_nlp/blob/master/readme_resource/2.jpg)\n',
    'author': 'CofinCup',
    'author_email': '864341840@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
